/*
 * Copyright (C) September 13, 2008 and on
 * Maciej Żenczykowski (zenczykowski@gmail.com)
 *
 * Site Reliability Engineer
 * Google, Inc. (maze@google.com)
 *
 * PhD Student
 * Algorithmics Research Group
 * Theoretical Computer Science
 * Faculty of Mathematics and Computer Science
 * Jagiellonian University (maze@tcs.uj.edu.pl)
 */

#ifndef _FS_JUNKFS_DEBUG_H
#define _FS_JUNKFS_DEBUG_H

static void printk_void(void) __attribute__ ((unused));
static void printk_void(void) { }

static void printk_int (int v)  { printk("%d", v); }
static void printk_long(long v) { printk("%ld",v); }

#define PK(_s,_t,_v)                                                do { printk(_s "%s: (%s)", __stringify(_v), __stringify(_t)); printk_ ## _t (_v); } while(0)

#define DBG_ENTER_pre                                               printk(KERN_DEBUG "@" __FILE__ ":" __stringify(__LINE__) " - Entering: %s(", __FUNCTION__)
#define DBG_ENTER_post                                              printk(")\n")

#define DBG_ENTER0()                                                do { DBG_ENTER_pre;                                                                                                         DBG_ENTER_post; } while(0)
#define DBG_ENTER1(_t1,_n1)                                         do { DBG_ENTER_pre; PK(,_t1,_n1);                                                                                           DBG_ENTER_post; } while(0)
#define DBG_ENTER2(_t1,_n1,_t2,_n2)                                 do { DBG_ENTER_pre; PK(,_t1,_n1); PK(", ",_t2,_n2);                                                                         DBG_ENTER_post; } while(0)
#define DBG_ENTER3(_t1,_n1,_t2,_n2,_t3,_n3)                         do { DBG_ENTER_pre; PK(,_t1,_n1); PK(", ",_t2,_n2); PK(", ",_t3,_n3);                                                       DBG_ENTER_post; } while(0)
#define DBG_ENTER4(_t1,_n1,_t2,_n2,_t3,_n3,_t4,_n4)                 do { DBG_ENTER_pre; PK(,_t1,_n1); PK(", ",_t2,_n2); PK(", ",_t3,_n3); PK(", ",_t4,_n4);                                     DBG_ENTER_post; } while(0)
#define DBG_ENTER5(_t1,_n1,_t2,_n2,_t3,_n3,_t4,_n4,_t5,_n5)         do { DBG_ENTER_pre; PK(,_t1,_n1); PK(", ",_t2,_n2); PK(", ",_t3,_n3); PK(", ",_t4,_n4); PK(", ",_t5,_n5);                   DBG_ENTER_post; } while(0)
#define DBG_ENTER6(_t1,_n1,_t2,_n2,_t3,_n3,_t4,_n4,_t5,_n5,_t6,_n6) do { DBG_ENTER_pre; PK(,_t1,_n1); PK(", ",_t2,_n2); PK(", ",_t3,_n3); PK(", ",_t4,_n4); PK(", ",_t5,_n5); PK(", ",_t6,_n6); DBG_ENTER_post; } while(0)

#define DBG_RETURN_pre                                              printk(KERN_DEBUG "@" __FILE__ ":" __stringify(__LINE__) " - Returning: %s(...)", __FUNCTION__)
#define DBG_RETURN_post                                             printk("\n")

#define DBG_RETURN0()                                               do {                DBG_RETURN_pre; printk(" = void");                                                             DBG_RETURN_post; return;     } while(0)
#define DBG_RETURN1(_t,_v)                                          do { _t _v_ = (_v); DBG_RETURN_pre; printk(" = %s = (%s)", __stringify(_v), __stringify(_t)); printk_ ## _t (_v_); DBG_RETURN_post; return _v_; } while(0)

#define DBG_MARK_pre                                                printk(KERN_DEBUG "@" __FILE__ ":" __stringify(__LINE__) " - Mark in %s(...)", __FUNCTION__)
#define DBG_MARK_post                                               printk("\n")

#define DBG_MARK0()                                                 do { DBG_MARK_pre;                                                                                                       DBG_MARK_post; } while(0)
#define DBG_MARK1(_t1,_n1)                                          do { DBG_MARK_pre; PK(" ",_t1,_n1);                                                                                      DBG_MARK_post; } while(0)
#define DBG_MARK2(_t1,_n1,_t2,_n2)                                  do { DBG_MARK_pre; PK(" ",_t1,_n1); PK(" ",_t2,_n2);                                                                     DBG_MARK_post; } while(0)
#define DBG_MARK3(_t1,_n1,_t2,_n2,_t3,_n3)                          do { DBG_MARK_pre; PK(" ",_t1,_n1); PK(" ",_t2,_n2); PK(" ",_t3,_n3);                                                    DBG_MARK_post; } while(0)
#define DBG_MARK4(_t1,_n1,_t2,_n2,_t3,_n3,_t4,_n4)                  do { DBG_MARK_pre; PK(" ",_t1,_n1); PK(" ",_t2,_n2); PK(" ",_t3,_n3); PK(" ",_t4,_n4);                                   DBG_MARK_post; } while(0)
#define DBG_MARK5(_t1,_n1,_t2,_n2,_t3,_n3,_t4,_n4,_t5,_n5)          do { DBG_MARK_pre; PK(" ",_t1,_n1); PK(" ",_t2,_n2); PK(" ",_t3,_n3); PK(" ",_t4,_n4); PK(" ",_t5,_n5);                  DBG_MARK_post; } while(0)
#define DBG_MARK6(_t1,_n1,_t2,_n2,_t3,_n3,_t4,_n4,_t5,_n5,_t6,_n6)  do { DBG_MARK_pre; PK(" ",_t1,_n1); PK(" ",_t2,_n2); PK(" ",_t3,_n3); PK(" ",_t4,_n4); PK(" ",_t5,_n5); PK(" ",_t6,_n6); DBG_MARK_post; } while(0)

#endif /* _FS_JUNKFS_DEBUG_H */
